from pydantic import BaseModel

from popug_sdk.schemas.response_schema import get_response_schema


class Hello(BaseModel):
    hello: str | None = None


if __name__ == "__main__":
    schema = get_response_schema(Hello)
    res = schema(**{"result": {"hello": "world"}})
    print(res.json())



from typing import Any

import jwt
from fastapi import (
    Depends,
    HTTPException,
)
from jwt import PyJWTError
from starlette import status

from popug_sdk.auth.oauth_schema import oauth2_schema
from popug_sdk.conf import settings
from popug_sdk.conf.auth import AuthSettings


def get_token_data(token: str = Depends(oauth2_schema)) -> dict[str, Any]:
    token = token.replace("Bearer ", "")

    auth_settings: AuthSettings = settings.auth

    try:
        token_payload: dict[str, Any] = jwt.decode(
            token,
            auth_settings.public_key,
            algorithms=[auth_settings.algorithm],
        )
    except PyJWTError:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED, detail="Invalid token"
        )

    return token_payload



from typing import (
    TYPE_CHECKING,
    Callable,
)

from popug_sdk.redis.redis_pool import get_redis_pool

if TYPE_CHECKING:
    from redis import Redis as BaseRedis

    Redis = BaseRedis[bytes]
else:
    from redis import Redis


def get_redis(config_name: str = "default") -> Callable[[], Redis]:
    def _get_redis() -> Redis:
        return Redis(connection_pool=get_redis_pool(config_name))

    return _get_redis



import logging.config
from typing import Any

from pydantic import (
    BaseModel,
    BaseSettings,
    Field,
)

from popug_sdk.conf.amqp import AMQPConfigSettings
from popug_sdk.conf.auth import AuthSettings
from popug_sdk.conf.constants import (
    BASE_NAME,
    LOCALHOST,
    LOG_LEVEL,
    PortType,
)
from popug_sdk.conf.db import DatabaseSettings
from popug_sdk.conf.redis import RedisConfigSettings


class AppSettings(BaseModel):
    debug: bool = False
    instance: str = "src.main:app"
    app_schema: str = "http"
    app_host: str = LOCALHOST
    app_port: PortType = "8080"


class AlembicSettings(BaseModel):
    config: str = "src/alembic_config/alembic.ini"
    directory: str = "src/alembic_config"


class Settings(BaseSettings):
    project: str = BASE_NAME
    version: str = "0.0.1"
    debug: bool = False
    use_https: bool = True
    log_level: str = LOG_LEVEL
    tag_groups: list[dict[str, Any]] = Field(default_factory=list)
    commands_roots: list[str] = Field(default_factory=lambda: ["src"])

    app: AppSettings = Field(default_factory=AppSettings)
    alembic: AlembicSettings = Field(default_factory=AlembicSettings)
    database: DatabaseSettings = Field(default_factory=DatabaseSettings)
    amqp: AMQPConfigSettings = Field(default_factory=AMQPConfigSettings)
    redis: RedisConfigSettings = Field(default_factory=RedisConfigSettings)
    redis_configs: list[str] = Field(default_factory=list)

    auth: AuthSettings = Field(default_factory=AuthSettings)

    @property
    def database_dsn(self) -> str:
        database_dsn_template = (
            "postgresql://"
            "{database_user}:{database_password}@"
            "{database_host}:{database_port}/"
            "{database_name}"
        )

        return database_dsn_template.format(
            database_user=self.database.user,
            database_password=self.database.password,
            database_host=self.database.host,
            database_port=self.database.port,
            database_name=self.database.database_name,
        )

    @property
    def logging(self) -> dict[str, Any]:
        return {
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {
                "consoleFormatter": {
                    "()": "uvicorn.logging.DefaultFormatter",
                    "fmt": "[%(process)d] [%(asctime)s] [%(levelname)s] %(name)s -> %(message)s",  # noqa
                },
                "access": {
                    "()": "uvicorn.logging.AccessFormatter",
                    "fmt": "[%(asctime)s] [%(levelname)s] %(name)s -> '%(request_line)s' %(status_code)s",  # noqa
                },
            },
            "handlers": {
                "console": {
                    "level": "DEBUG",
                    "class": "logging.StreamHandler",
                    "stream": "ext://sys.stdout",
                    "formatter": "consoleFormatter",
                },
                "access": {
                    "formatter": "access",
                    "class": "logging.StreamHandler",
                    "stream": "ext://sys.stdout",
                },
            },
            "loggers": {
                self.project: {
                    "handlers": ["console"],
                    "level": self.log_level,
                },
                "uvicorn.error": {"level": "INFO"},
                "uvicorn": {
                    "handlers": ["access"],
                    "level": "INFO",
                    "propagate": False,
                },
            },
        }

    class Config:
        env_prefix = f"{BASE_NAME.upper()}_"
        env_file = ".env"
        env_nested_delimiter = "__"

    def set_up_logging(self) -> None:
        logging.config.dictConfig(self.logging)



import typer
from alembic.command import init
from alembic.config import Config

from popug_sdk.conf import settings


def init_alembic(directory: str = typer.Option(None)) -> None:
    directory = directory or settings.alembic.directory
    init(Config(settings.alembic.config), directory)



from enum import Enum

from pydantic import (
    BaseModel,
    Field,
)

from popug_sdk.conf.constants import (
    LOCALHOST,
    PortType,
)


class ExchangeType(Enum):
    direct = "direct"
    fanout = "fanout"
    headers = "headers"
    topic = "topic"


class ExchangeSettings(BaseModel):
    type: ExchangeType = ExchangeType.topic
    name: str | None = None
    durable: bool = True


class QueueSettings(BaseModel):
    name: str | None = None


class AMQPSettings(BaseModel):
    host: str = LOCALHOST
    port: PortType = 5672
    username: str = "guest"
    password: str = "guest"
    virtual_host: str = "/"
    exchange: ExchangeSettings = Field(default_factory=ExchangeSettings)
    queue: QueueSettings = Field(default_factory=QueueSettings)
    routing_keys: list[str] = Field(default_factory=list)

    @property
    def url(self) -> str:
        amqp_url_template = (
            "amqp://{user}:{password}" "@{host}:{port}{virtual_host}"
        )
        return amqp_url_template.format(
            username=self.username,
            password=self.password,
            host=self.host,
            port=self.port,
            virtualhost=self.virtual_host,
        )


class AMQPConfigSettings(BaseModel):
    default: AMQPSettings = Field(default_factory=AMQPSettings)



import logging
from functools import partial
from typing import Any

import pika
from pika import spec
from pika.channel import Channel

from popug_sdk.amqp.utils import get_connection_params
from popug_sdk.conf import settings
from popug_sdk.conf.amqp import AMQPSettings

logger = logging.getLogger(settings.project)


class BaseConsumer:
    TIMEOUT = 5

    def __init__(self, config: AMQPSettings):
        self._config = config

        self._connection: pika.SelectConnection | None = None
        self._channel: Channel | None = None
        self._closing: bool = False
        self._consumer_tag: str | None = None

    def _connect(self) -> pika.SelectConnection:
        parameters = get_connection_params(self._config)

        logger.info(
            f"Connecting to host={parameters.host} port={parameters.port} "
            f"virtual_host={parameters.virtual_host}>"
        )
        return pika.SelectConnection(parameters, self._on_connection_open)

    def _on_connection_open(self, connection: pika.SelectConnection) -> None:
        logger.info("Connection opened")

        connection.add_on_close_callback(self._on_connection_closed)
        connection.channel(on_open_callback=self._on_channel_open)

    def _on_connection_closed(
        self, connection: pika.SelectConnection, reason: Exception
    ) -> None:
        if self._closing:
            self._connection.ioloop.stop()  # type: ignore
        else:
            logger.warning(
                f"Connection closed, "
                f"reopen in {self.TIMEOUT} seconds: {reason}"
            )

            connection.add_timeout(self.TIMEOUT, self._reconnect)

    def _reconnect(self) -> None:
        if self._connection:
            self._connection.ioloop.stop()

        if not self._closing:
            self._connection = self._connect()
            self._connection.ioloop.start()

    def _on_channel_open(self, channel: Channel) -> None:
        logger.info("Channel opened")
        self._channel = channel
        self._channel.add_on_close_callback(self._on_channel_close_ok)

        self._setup_exchange(self._channel)

    def _on_channel_close_ok(self, channel: Channel, _: Any) -> None:
        channel.connection.close()

    def _setup_exchange(self, channel: Channel) -> None:
        exchange_name = self._config.exchange.name
        logger.info(f"Declaring exchange {exchange_name}")

        channel.exchange_declare(
            exchange=exchange_name,
            exchange_type=self._config.exchange.type.value,
            callback=partial(self._setup_queue, channel),
            durable=self._config.exchange.durable,
        )

    def _setup_queue(self, channel: Channel, _: Any) -> None:
        queue_name = self._config.queue.name
        logger.info("Declaring queue %s", queue_name)

        channel.queue_declare(
            queue=queue_name,
            callback=partial(self._on_queue_declareok, channel),
        )

    def _on_queue_declareok(self, channel: Channel, _: Any) -> None:
        for routing_key in self._config.routing_keys:
            logger.info(
                f"Binding {self._config.exchange.name} to "
                f"{self._config.queue.name} with {routing_key}",
            )
            channel.queue_bind(
                queue=self._config.queue.name,
                exchange=self._config.exchange.name,
                routing_key=routing_key,
                callback=partial(self._start_consuming, channel),
            )

    def _start_consuming(self, channel: Channel, _: Any) -> None:
        logger.info("Issuing consumer related RPC commands")
        channel.add_on_cancel_callback(channel.close)
        self._consumer_tag = channel.basic_consume(
            queue=self._config.queue.name,
            on_message_callback=self.on_message,
        )

    def on_message(
        self,
        channel: Channel,
        basic_deliver: spec.Basic.Deliver,
        properties: spec.BasicProperties,
        body: bytes,
    ) -> None:
        logger.debug(
            f"Received message # {basic_deliver.delivery_tag} "
            f"from {properties.app_id}: {body!r}"
        )
        self.process_message(basic_deliver, properties, body)
        channel.basic_ack(basic_deliver.delivery_tag)

    def process_message(
        self,
        basic_deliver: spec.Basic.Deliver,
        properties: spec.BasicProperties,
        body: bytes,
    ) -> None:
        pass

    def run(self) -> None:
        self._connection = self._connect()
        self._connection.ioloop.start()

    def stop(self) -> None:
        logger.info("Stopping")
        self._closing = True

        if self._channel:
            self._channel.basic_cancel(
                consumer_tag=self._consumer_tag,
                callback=lambda _: self._channel.close,
            )
        self._connection.ioloop.start()  # type: ignore
        logger.info("Stopped")



from __future__ import annotations

from typing import (
    Generic,
    TypeVar,
)

from sqlalchemy.orm import Session

RepoData = TypeVar("RepoData")
RepoT = TypeVar("RepoT", bound="BaseRepo")  # type: ignore


class NoContextError(Exception):
    pass


class BaseRepo(Generic[RepoData]):
    def __init__(
        self,
        session: Session,
        context: RepoData | None = None,
    ):
        self._session = session
        self._context = context

    def __call__(self, context: RepoData | None) -> RepoT:
        self._context = context

        return self  # type: ignore

    @property
    def is_empty(self) -> bool:
        return self._context is None

    def status(self) -> RepoData:
        self._session.flush()

        return self.get()

    def apply(self) -> RepoData:
        result = self.get()
        self._session.commit()

        return result

    def get(self) -> RepoData:
        if self._context is None:
            raise NoContextError

        return self._context

    def context(self) -> RepoData | None:
        return self._context
